from .auth import AuthMixin  # noqa
from .email import EmailMixin  # noqa
from .player import PlayerMixin  # noqa
from .server import ServerMixin  # noqa
from .recommender import RecommenderMixin  # noqa



__version__ = '2.0.0'
import pathlib  # isort:skip
root = pathlib.Path(__file__).parent  # isort:skip

import kick  # isort:skip
kick.start('spfy')  # isort:skip

from kick import config, logger  # isort:skip

from .result import SpotifyResult
from .client import SpotifyClient
from .wrapper import Spotify
from .constants import *
from .exceptions import *



import inspect

import hug
from gunicorn.app.base import BaseApplication


class StandaloneApplication(BaseApplication):
    def __init__(self, app, **options):
        self.options = options
        self.application = app
        super().__init__()

    def load_config(self):
        config = {
            k: v for k, v in self.options.items()
            if k in self.cfg.settings and v is not None
        }
        for key, value in config.items():
            self.cfg.set(key.lower(), value)

    def load(self):
        return self.application


class ServerMixin:
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def runserver(self, **gunicorn_options):
        for name, method in inspect.getmembers(self, inspect.ismethod):
            hug.get(f'/{name}')(method)

        from ..client import __hug__ as client_hug
        __hug__.extend(client_hug)  # noqa

        app = StandaloneApplication(__hug_wsgi__, **gunicorn_options)  # noqa
        app.run()



#!/usr/bin/env python3
import fire

from . import config
from .client import SpotifyClient
from .mixins import PlayerMixin, ServerMixin, RecommenderMixin


class Spotify(SpotifyClient, PlayerMixin, RecommenderMixin, ServerMixin):
    """Spotify high-level wrapper."""
    cli = False

    def __init__(self, email=None, username=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.email = email or config.auth.email
        self.username = username or config.auth.username
        if self.cli and not self.is_authenticated:
            self.authenticate(email=self.email, username=self.username)

    def __dir__(self):
        names = super().__dir__()
        names = [name for name in names if not name.startswith('_') and name != 'user']
        return names


def main():
    """Main function."""

    try:
        Spotify.cli = True
        fire.Fire(Spotify)
    except KeyboardInterrupt:
        print('Quitting')


if __name__ == '__main__':
    main()



import addict
from cached_property import cached_property


class SpotifyResult(addict.Dict):
    ITER_KEYS = ('items', 'artists', 'tracks', 'albums', 'audio_features')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def __iter__(self):
        for key in self.ITER_KEYS:
            if key in self:
                return iter(self[key])
        return super().__iter__()

    @cached_property
    def all(self):
        return [item for item in self.iterall()]

    @cached_property
    def next(self):
        if self['next']:
            return self.client._get(self['next'])
        else:
            return None

    def iterall(self):
        result = self

        while result:
            for item in result:
                yield item
            result = result.next
